# Generated by Django 2.2.15 on 2020-08-21 14:05

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="GARInstitution",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "uai",
                    models.CharField(
                        max_length=14,
                        unique=True,
                        verbose_name="Unité Administrative Immatriculée",
                    ),
                ),
                (
                    "institution_name",
                    models.CharField(
                        max_length=255, verbose_name="Nom de l'institution"
                    ),
                ),
                (
                    "ends_at",
                    models.DateField(
                        null=True, verbose_name="Date de fin d'abonnement"
                    ),
                ),
                (
                    "user",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
    ]
