# Generated by Django 3.2.16 on 2023-01-03 13:54

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("django_gar", "0004_garsession"),
    ]

    operations = [
        migrations.AddField(
            model_name="garinstitution",
            name="project_code",
            field=models.CharField(
                blank=True,
                max_length=50,
                null=True,
                verbose_name="Code de projet ressources",
            ),
        ),
    ]
